package ru.dbotthepony.kstarbound.world.entities

import com.google.gson.JsonElement
import com.google.gson.JsonObject
import ru.dbotthepony.kstarbound.client.render.RenderLayer
import ru.dbotthepony.kstarbound.defs.actor.PersistentStatusEffect
import ru.dbotthepony.kstarbound.math.vector.Vector2d
import ru.dbotthepony.kstarbound.world.Direction

interface IAnchorState {
	val position: Vector2d
	val exitBottomPosition: Vector2d?
	val direction: Direction
	val angle: Double
}

data class AnchorState(
	override val position: Vector2d,
	override val exitBottomPosition: Vector2d?,
	override val direction: Direction,
	override val angle: Double
) : IAnchorState

// int32_t
enum class LoungeOrientation(val humanoidState: HumanoidActorEntity.HumanoidState) {
	NONE(HumanoidActorEntity.HumanoidState.IDLE),
	SIT(HumanoidActorEntity.HumanoidState.SIT),
	LAY(HumanoidActorEntity.HumanoidState.LAY),
	STAND(HumanoidActorEntity.HumanoidState.IDLE);
}

data class LoungeAnchorState(
	override val position: Vector2d,
	override val exitBottomPosition: Vector2d?,
	override val direction: Direction,
	override val angle: Double,
	val orientation: LoungeOrientation = LoungeOrientation.NONE,
	val loungeRenderLayer: RenderLayer,
	val controllable: Boolean = false,
	val statusEffects: List<PersistentStatusEffect> = listOf(),
	val effectEmitters: Set<String> = setOf(),
	val emote: String? = null,
	val dance: String? = null,
	val directives: String? = null,
	val armorCosmeticOverrides: Map<String, JsonElement> = mapOf(),
	val cursorOverride: String? = null,
	val cameraFocus: Boolean = false,
) : IAnchorState
